********************************************************************************
*PART I Getting started / Data Management  Part I / Patterns of missing values*
********************************************************************************

* ### 1 Getting started ####
********************************************************************************
********************************************************************************

*** Outline

* # 1 Getting started 
* # 1.1 Statas Documentation and Help Files
* # 1.2 Searching for Information
* # 1.3 Command grammar
* # 1.4 Saving and printing outputlog files
* # 1.5 Do files
* # 1.6 Local and global macros
* # 1.6.1 Local macros
* # 1.6.2 Global macros
* # 1.7 An Example Stata Session

********************************************************************************
* ### 1 Getting started ###

********************************************************************************
********************************************************************************
* ### 1.1 Statas Documentation and Help Files ###

*1.1.1 STATA manual
*(a) Menu: Help > PDF Documentation
*(b) Command: help documentation. Type "help documentation" for more information.

*1.1.2 Books
*Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire
*Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press

*1.1.3. The help command is particularly useful when used with a command name. 
// Typing "help correlate", for example, causes a description of that command to appear in a Viewer window.


********************************************************************************
********************************************************************************
* ### 1.2 Searching for Information ###

*(a) Menu: Help > Search > Search documentation and FAQs 
// provides a direct way to search for information in Statas documentation 
// or in the websites FAQs and other pages. 
*(b) Command: search // The "search" command can do similar things. Type "help search" for more about this command.


********************************************************************************
********************************************************************************
* #### 1.3 Command grammar ###

// the structure of commands is some subset of following elements

* [ prefix:] command [varlist] [ = exp] [if exp] [in range] [weight] [using filename] [, options]

// elements displayed in brackets are optional; elements without brackets are required 

* #### Brief description ###

* Square brackets distinguish optional qualifiers and options from required ones

* # "command" is the name of the command that should describe what it does;
* # "varlist" is the list of variable names seperated by spaces;
* # "prefix:" examples:
* ## by varlist[, options]: "by varlist, sort:" prefix makes Stata sort the dataset accoring to the variable in the varlist 
// and repeat a command for each subset of the data for which the values of the variables in varlist are equal.
* # "=exp" specifies the value to be assigned to a variable and is most often used with "generate" and "replace".
* # "if exp" qualifier restricts the scope of a command to those observations for which the value of the expression is true.
* # "in range" qualifier means that the command is to use only the observations specified
// "list price in -10/1" says to list the last 10 observations.
* # "weight" indicates the weight to be attached to each observation.
* ## fweights, or frequency weights, indicate duplicated observations.
* ## pweights, or sampling weights, denote the inverse of the probability that this observation is included in the sample because of the sampling design.
* ## aweights, or analytic weights, are inversely proportional to the variance of an observation.
* ## iweights, or importance weights, indicate the relative importance of the observation.
* # "options" denotes a list of options.


********************************************************************************
********************************************************************************
* ### 1.4 Saving and printing outputlog files ###

// The best way to prepare for recording the session is by opening a log file at the start.
// Log files contain commands and results tables, but not graphs.

* Commands I
* # To start a log: log using filename
* # To temporarily stop logging:
* ## Temporarily stop: log off
* ## Resume: log on
* # To stop logging and close the file: log close
* # To create "filename.log" that you can load into your word processor: translate filename.smcl filename.log
* # To create a PDF of filename.smcl on Windows or Mac: translate filename.smcl filename.pdf

* Commands II
* # append to an existing log: log using filename, append
* # replace an existing log: log using filename, replace

* Using the GUI (Graphical User Interface):
* # To start a log: click on the Log button
* # To temporarily stop logging: click on the Log button, and choose Suspend
* # To resume: click on the Log button, and choose Resume
* # To stop logging and close the file: click on the Log button, and choose Close
* # To print previous or current log: select File > View, choose file, right-click on the Viewer, and select Print

* Also, "cmdlog" will produce logs containing solely what you typedlogs that, although not containing your results, are sufficient to recreate the session.
// These files are perfect for later going back and creating a Stata do-file.
* # To start a command-only log: cmdlog using filename
* # To stop logging and close the file: cmdlog close
* # To recreate your session: do filename.txt

* Creating the Do files is an alternative way to recreate a session


********************************************************************************
********************************************************************************
* ### 1.5 Do files ###

* Creating and running dofiles
* # 1. You can execute do-files by typing do followed by the filename.
* # 2. You can execute do-files by selecting File > Do....
* # 3. You can use the Do-file Editor to compose, save, and execute do-files
// To use the Do-file Editor, click on the Dofile Editor button, or type doedit in the Command window.

// important part of directly created do files are comments; these are intiated by:
	// 1) * at the beginign of line (anything on that line after is comment); note that if anything 
	// 		precedes the star, the star will be considered as multiplication sign (changes its meaning)
	// 2) // anywhere on the line; anything on given line after // is considered a comment
	// 3) /* in beginning and */ at the end; this allows for commands spanning more lines. /* */ can be put anywhere, even in the middle of a line
	
// line breaks are issued by "enter" in do files; we can change this by the command "#delimit"
// We can change the delimiter back to carriage return by typing #delimit cr.

// if command is to long, alternative is to use ///, which means Stata ignores the end of a line,
// and naturally continues reading on following line. Alternatively, use /* at the end of the first line 
// and */ at the beginning of the second line, which both have to be recognized as one line.

// usually we start the do file by set of commands; clear all (to clear the data from memory);
// version number (to help deal with changing command names over versions); set more off
// (to deactivate partitioned display of output)


version 11.0
clear all
set more off

********************************************************************************
********************************************************************************
* ### 1.6 Local and global macros ###

* [based on Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire, Ch. 14]

* 1.6.1 Local macros

* Macros are names (up to 31 characters) that can stand for strings, program-defined numerical results or user-defined values. 
// A local macro exists only within the program that defines it, and cannot be referred to in another program. 
// To create a local macro named "iterate", standing for the number 0, type

local iterate 0

// To refer to the contents of a local macro (0 in this example), place the macro name within left and right single quotes.

display `iterate'

// Thus, to increase the value of iterate by one, we write 

local iterate = `iterate' + 1
display `iterate'

* 1.6.2 Global macros

* Once defined, global macros  remain in memory and can be used by other programs for the duration of your current Stata session. 
// To refer to a global macros contents, we preface the macro name with a dollar sign

global distance = 73
display $distance * 2

 * Define path to files used during the seminar

global path_o="D:\Study\CERGE-EI\TA\Stata_seminar\Datasets_original"
global path_w="D:\Study\CERGE-EI\TA\Stata_seminar\Datasets_working"

/*
If you enter filename without path, Stata automatically looks in the current directory
There are several general commands related to working directory:

pwd // dispalys present working directory
dir // shows list of files that are stored in current folder
cd // changes directory, from which is stata working and where it is saving files

Instead of setting the path, we could change current directory:
cd "D:\Study\CERGE-EI\TA\Stata_seminar\Datasets_original"
*/


********************************************************************************

* ### 1.7 An Example Stata Session ###

* [based on Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire, Ch. 1]

* Given small time series, which covers satellite-era (1979 to 2011) observations of ice on the Arctic Ocean in September
* Variables:
* # "extent" is a satellite-based measure of the Northern Hemisphere sea area with at least 15% ice concentration each September
* # "area" numbers represent the area of sea ice itself 
* # "tempN" describes mean annual surface air temperature above 64N latitude. 
// Temperatures are expressed as anomalies, which are deviations from the 19511980 average, in degrees Celsius. 
* 33 observations (years) and 8 variables.

*Opening a log file

log using $path_o\session1.smcl, replace
// Alternative ways:
* # File > Log > Begin ... from the top menu bar, and specify a name and folder for the resulting log file.
//After selecting which type of log file you want, click Save.
* # Use Log button

// Log files can be created either in a special Stata format (.smcl), or in ordinary text or ASCII format (.log). 
// A .smcl (Stata markup and control language) file will be nicely formatted for viewing or printing within Stata. 
// It could also contain hyperlinks that help to understand commands or error messages. 
// .log (text) files lack such formatting, but are simpler to use if you plan later to insert or edit the output in a word processor.

*Opening an existing Stata-format dataset named "Arctic9.dta"
use $path_o\Arctic9.dta, clear
// Alternative ways:
* # Select File > Open > Arctic9.dta using the top menu bar
* # Use Open button

* A brief description of the dataset now in memory
describe
// Alternative ways:
* # Many Stata commands can be abbreviated to their first few letters. We could shorten "describe" to just the letter "d". 
* # Data > Describe data > Describe data in memory > (OK).

* List the first 10 observations (years)
list in 1/10
// Alternative ways:
* # Shorten to the letter "l" -> "l in 1/10"
* # Data > Describe data > List data > (OK)

* Summary statistics
summarize
// Alternative ways:
* # Shorten to the letters "su"
* # Statistics > Summaries, tables, and tests > Summary and descriptive statistics > Summary statistics > (OK)
 
* To print results 
* # File > Print > Results
* Copy results to your word processor. Change the pasted text to a fixed-width font such as Courier.
 
* Correlations among variables
correlate extent area volume tempN
// Alternative ways:
* # Statistics > Summaries, tables, and tests > Summary and descriptive statistics > Correlation and covariance
// Then choose the variables to be correlated.

// September sea ice extent, area and volume all have strong positive correlations. 
// Their correlation with annual air temperature is negative: the warmer the air, the less ice (or vice versa). 

* Plot "extent" against "year"
graph twoway connect extent year
// The first-named variable in this command, extent, defines the vertical or y axis; the last-named variable, year, defines the horizontal or x axis.

// We see an uneven but steepening downward pattern, as September sea ice extent declined by more than a third over this period.

* To print graph, go to the Graph window and click its "print icon" or File > Print.
* To copy the graph  into a word processor, right-click on the graph, and select Copy Graph. 
* To save the graph either right-click and Save Graph, click "save icon" in the Graph window, 
// or select File > Save As from the Graph windows top menu bar.

* View log file session1.smcl
* # File > Log > View > OK

* Print the log file by clicking the icon on the top bar of the log files Viewer window. 

* Log files close automatically at the end of a Stata session, or earlier if instructed by
// Close log file, typing the command "log close", or by choosing File > Log > Close
log close
* To create an output file that can be opened easily by your word processor, either 
// translate the log file from .smcl (a Stata format) to .log (standard ASCII text format) by typing
translate $path_o\session1.smcl $path_o\session1.log, replace
// or start out by creating the file in .log instead of .smcl format either by specifying that you want the log in text format:
// "log using session1, text"
// or by specifying that the file to be created is a .log file:
// "log using session1.log"

********************************************************************************
********************************************************************************
********************************************************************************

* ### 2 Data Management - Part I ###
********************************************************************************
********************************************************************************

*** Outline

* #  2 Data Management - Part I
* #  2.1 Creating New Datasets
* #  2.1.1 Creating a New Dataset by Typing in Data
* #  2.1.2 Creating a New Dataset by Copy and Paste
* #  2.1.3 Importing Data from Other Programs
* #  2.2 Combining Two or More Stata Files
* #  2.3 Collapsing Data [important extra topic]
* #  2.4 Specifying Subsets of the Data: in and if Qualifiers [important extra topic]
* #  2.5 Generating and Replacing Variables [important extra topic]
* #  2.6 Missing Value Codes
* #  2.7 Using functions
* #  2.8 Converting Between Numeric and String Formats
* #  2.9 Using Explicit Subscripts with Variables

********************************************************************************
********************************************************************************
* ### 2 Data Management - Part I ###

* Define path to files used during the seminar

global path_o="D:\Study\CERGE-EI\TA\Stata_seminar\Datasets_original"
global path_w="D:\Study\CERGE-EI\TA\Stata_seminar\Datasets_working"

* [based on Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire, Ch. 2]

********************************************************************************
********************************************************************************
* ### 2.1 Creating New Datasets ###

* ### 2.1.1 Creating a New Dataset by Typing in Data ###

* To create a dataset from printed information like Table 2.1 is through the 
* # Data Editor, invoked by clicking respectide Button
* # selecting Window > Data Editor from the menu bar
* # Typing the command "edit"

// We can assign more descriptive variable names by double-clicking on the column headings (such as var1) and then typing a new name in the resulting dialog box; 
// Eight characters or fewer works best,  names with up to 32 characters are allowed. 
// We can also create variable labels that contain a brief description. 
// For example, var2 (population) might be renamed pop, and given the variable label Population in 1000s, 1995.

// Renaming and labeling variables can also be done outside of the Data Editor through the rename and label variable commands:
* # "rename var1 place"
* # "rename var2 pop"
* # "rename var3 unemp"
* # "rename var4 mlife"
* # "rename var5 flife"

* Alternatively, the five rename operations could be accomplished in one step:
// "rename (var1 var2 var3 var4 var5) (place pop unemp mlife flife)"

* # "label data "Canadian dataset 1""
* # "label variable place "Place name"
* # "label variable pop "Population in 1000s, 1995""
* # "label variable unemp "% 15+ population unemployed, 1995""
* # "label variable mlife "Male life expectancy years""
* # "label variable flife "Female life expectancy years""

// Numbers should not include any commas!
// String variable values can be almost any combination of letters, numbers, symbols or spaces up to 244 characters long.


* ### 2.1.2 Creating a New Dataset by Copy and Paste ###

* Given Canada1.xlsx


********************************************************************************
********************************************************************************
* ### 2.1.3 Importing Data from Other Programs ###

* # import excel

* Stata can import data from Excel spreadsheets directly through menu selections
* # File > Import > Excel spreadsheet (*.xls; *.xlsx)
* # or the "import excel" command. 

// Spreadsheets often contain titles, notes, subtables, multiple sheets, graphs or other features
// To restrict the import operation to a particular range of cells, use a "cellrange()" option. 
// The "sheet()" option can specify what sheet within the spreadsheet to import. 
// A "firstrow" option tells Stata that the first row of these cells contain variables names. 

* Given snowfall.xls
import excel using $path_o\snowfall.xls, sheet("Berlin") cellrange(a4:o56) firstrow clear
describe

* To meet Stata criteria 
* # Start variable names with a letter or underscore, and have no embedded blanks. 
* # Missing values should be replaced with blanks or numerical codes, 
// and non-numeric characters removed from cells in columns meant to represent numerical variables.

* # insheet

* Text file MEI.raw contains monthly values of MEI, Multivariate ENSO Index (MEI) describing tropical Pacific conditions, from January 1950 through December 2011.
// These are tab-separated values
// The first row of the text file contains a list of variable names: mei1 for January MEI, mei2 for February, and so forth 
// The first few rows are shown in your handouts

// We can read these data into Stata using the insheet command, with options to specify that values are tab-separated, 
// and the first row contains variable names.

insheet using $path_o\MEI.raw, tab name clear
save $path_w\MEI0.dta, replace
describe

// With a comma option instead of tab, insheet could read a text file of comma-separated values,
// Text files can be read through Stata menus as well. Explore Data > Import to see the options available.

* # infix

// A different arrangement called fixed-column format has values that are not necessarily delimited at all, 
// but do occupy predefined column positions. The "infix" command can read such files. 
// We have to specify exactly how the columns should be read.

// Consider a text (ASCII) file named nfresour.raw in your handouts

// These data concern natural resource production in Newfoundland. 
// The four variables occupy fixed column positions: 
// * columns 14 are the years (1986...1991); 
// * columns 58 measure forestry production in thousands of cubic meters (2408...missing); 
// * columns 914 measure mine production in thousands of dollars (764,169...793,000); 
// * and columns 1518 are the consumer price index relative to 1986 (1000...1262). 

// Notice that in fixed-column format blanks indicate missing values. To read nfresour.raw into Stata:

infix year 1-4 wood 5-8 mines 9-14 CPI 15-18 using $path_o\nfresour.txt, clear
list

// Type "help import" to learn more about different methods:
// import excel, insheet, odbc, infile (free format), infix (fixed format), infile (fixed format), import sasxport, haver, xmluse

* To export data from Stata to other programs: 
// * File > Export > Excel spreadsheet (*.xls; *.xlsx)
// * Copy your data from Statas Data Editor or Data Browser and paste this directly into a spreadsheet such as Excel.
* The outsheet and outfile commands (or Data > Export) can write text files in several different formats. 


* #############################################
* ### 2.2 Combining Two or More Stata Files ###

* Combine Stata datasets in two general ways: 
* # "append" a second dataset that contains additional observations
* # "merge" with other datasets that contain new variables or values. 

* Append example

* File lakewin1.dta contains the ice-out dates for New Hampshires largest lake recorded from 1887 through 2007.

use $path_o\lakewin1.dta, clear
describe
list in -4/l

* File lakewin2.dta contains newer data from 2008 through 2012. 
// It has the same variables and format, so we can combine the update in lakewin2.dta with the older information in lakewin1.dta, using the "append" command.

use $path_o\lakewin2.dta, clear
describe
list

append using $path_o\lakewin1.dta

sort year
label data "Lake Winnipesaukee ice out 1887-2012"
save $path_w\lakewin3, replace
list in -7/l

* Merge example

* File lakesun.dta contains ice out dates for New Hampshires lake over the years 1869 through 2012. 
// Lake Sunapee (lakesun.dta) and Lake Winnipesaukee (lakewin3.dta) both form yearly series that could easily be combined into one dataset. 
// We do this with the "merge 1:1 year" command.

use $path_o\lakesun.dta, clear
sort year
describe
merge 1:1 year using $path_w\lakewin3.dta

label data "Sunapee & Winnipesaukee ice out 1869-2012"
describe
save $path_w\lakesunwin.dta, replace

// The merge results tell us that 126 years were present in both the master dataset (the data currently in memory  lakesun.dta in this example) 
// and the using dataset (lakewin3.dta).
// A further 18 years (1869 to 1886) existed only in lakesun.dta, so the Lake Winnipesaukee variables will have missing values in those years. 

* "merge" commands create a variable named _merge that records whether the observation came from:
* # the master data only (_merge = 1)
* # the using data only (_merge = 2)
* # both (_merge = 3)

// By default, whenever the same variables are found in both datasets, those of the master data are retained and those of the using data ignored. 

* "merge" options:
* # "merge 1:1 year using newdata.dta, update"
// A command would allow any missing values in the master data to be replaced by corresponding nonmissing values found in the using data (here, newdata.dta)

* # "l;merge 1:1 year using newdata, update replace"
// A command causes any values from the master data to be replaced by nonmissing values from the using data, if the latter are different

// Also possible are 1-to-many (1:m), many-to-1 (m:1) or many-to-many (m:m) operations. 
// Type "help merge" for details
// Merging and appending data can be accomplished through Data > Combine datasets menus as well.

*** "append" might be compared to lengthening a sheet of paper (that is, the dataset in memory) 
// by taping a second sheet with new observations (rows) to its bottom. 
*** "merge", in its simplest form, corresponds to widening our sheet of paper 
// by taping a second sheet to its right side, thereby adding new variables (columns)

********************************************************************************
********************************************************************************

